import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, max, format, extent } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import {Dropdown} from './Dropdown'


const width = 960;
const menuHeight = 70;
const height = 525- menuHeight;
const margin = { top: 20, right: 60, bottom: 65, left: 90 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;

// copy and paste the attributes constant up here
const attributes = [
    {value: 'Property Rights', label:'Property Rights'},
    {value: 'Judical Effectiveness',label: 'Judical Effectiveness'},
    {value: 'Government Integrity',label: 'Government Integrity' },
    {value: 'Tax Burden', label: 'Tax Burden'},
    {value: 'Government Spending', label:'Government Spending'},
    {value: 'Fiscal Health', label:'Fiscal Health' },
    {value: 'Business Freedom', label:'Business Freedom' },
    {value: 'Labor Freedom', label:'Labor Freedom' }

];


// do a linear search of the attributes constant using the for loop; 
// aviod array.prototype.find because it is not compatible for all browsers
const getLabel = value => {
	for (let i = 0; i < attributes.length; i++) {
		if(attributes[i].value === value){
    	return attributes[i].label;
    }
  }
};

const App = () => {
  const data = useData();
  
  // following code need to be moved before the condition
  const initialXAttribute = 'Judical Effectiveness';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);  
  const xValue = d => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);
  
  const initialYAttribute = 'Tax Burden';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);  
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);
  
  // conditional that  returns a loading indicator if the data is not loaded
  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
 	
  const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);


  return (
    <>
      <label for="x-select">X:</label>
      <Dropdown
        options = {attributes} 
        id = "x-select"
        selectedValue = {xAttribute}
        onSelectedValueChange={setXAttribute}  
      />
      
      <label for="y-select">Y:</label>
      <Dropdown
        options = {attributes} 
        id = "y-select"
        selectedValue = {yAttribute}
        onSelectedValueChange={setYAttribute}  
      />
      
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${innerHeight /
              2}) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
          <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={5} />
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>
          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            tooltipFormat={xAxisTickFormat}
            circleRadius={7}
          />
        </g>
      </svg> 
  	</>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
